package com.tourism.webadmin.app.website.controller;

import com.tourism.common.core.object.MyOrderParam;
import com.tourism.common.core.object.MyPageData;
import com.tourism.common.core.object.ResponseResult;
import com.tourism.common.core.object.TokenData;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.core.util.MyPageUtil;
import com.tourism.webadmin.back.dto.TourOrderDto;
import com.tourism.webadmin.back.model.TourOrder;
import com.tourism.webadmin.back.model.TourOrderPassenage;
import com.tourism.webadmin.back.model.TourismProject;
import com.tourism.webadmin.back.service.TourOrderPassenageService;
import com.tourism.webadmin.back.service.TourOrderService;
import com.tourism.webadmin.back.service.TourismProjectService;
import com.tourism.webadmin.back.vo.TourOrderVo;
import com.tourism.webadmin.back.vo.TourOrderPassenageVo;
import com.tourism.webadmin.back.vo.TourismProjectVo;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 个人订单控制器类。
 *
 * @author 吃饭睡觉
 * @date 2024-09-06
 */
@Tag(name = "我的订单管理接口")
@Slf4j
@RestController
@RequestMapping("/website/tourism/myOrder")
public class TourismOrderController {

    @Autowired
    private TourOrderService tourOrderService;
    @Autowired
    private TourOrderPassenageService tourOrderPassenageService;
    @Autowired
    private TourismProjectService tourismProjectService;

    /**
     * 列出符合过滤条件个人订单列表。
     *
     * @param tourOrderDto 旅游订单Dto对象
     * @return 应答结果对象，包含查询结果集。
     */
    @GetMapping("/list")
    public ResponseResult<MyPageData<TourOrderVo>> list(TourOrderDto tourOrderDto){
        TourOrder tourOder = new TourOrder();
        if(tourOrderDto.getOrderStatus() != null){
            tourOder.setOrderStatus(tourOrderDto.getOrderStatus());
        }
        TokenData tokenData = TokenData.takeFromRequest();
        tourOder.setCustomerMobile(tokenData.getLoginName());
        MyOrderParam myOrderParam = new MyOrderParam();
        myOrderParam.add(new MyOrderParam.OrderInfo("createTime",false,null));
        String orderBy = MyOrderParam.buildOrderBy(myOrderParam, TourOrder.class);
        List<TourOrder> tourOderList = tourOrderService.getTourOrderList(tourOder, orderBy);
        MyPageData<TourOrderVo> tourOderVoMyPageData = MyPageUtil.makeResponseData(tourOderList, TourOrderVo.class);

        List<TourOrderVo> dataList = tourOderVoMyPageData.getDataList();
        dataList.stream().forEach(item->{
            if(item.getProjectId() != null){
                TourismProject tourismProject = tourismProjectService.getById(item.getProjectId());
                TourismProjectVo tourismProjectVo = MyModelUtil.copyTo(tourismProject, TourismProjectVo.class);
                item.setTourismProjectVo(tourismProjectVo);
            }
        });
        return ResponseResult.success(tourOderVoMyPageData);
    }

    /**
     * 列出符合过滤条件个人订单详情。
     *
     * @param orderId 订单id
     * @return 应答结果对象，包含查询结果集。
     */
    @GetMapping("/detail")
    public ResponseResult<TourOrderVo> detail(@RequestParam Long orderId){
        TourOrder tourOder = new TourOrder();
        tourOder.setId(orderId);
        TokenData tokenData = TokenData.takeFromRequest();
        tourOder.setCustomerMobile(tokenData.getLoginName());
        TourOrder tourOder1 = tourOrderService.getOne(tourOder);
        TourOrderPassenage tourOrderPassenage = new TourOrderPassenage();
        tourOrderPassenage.setOrderId(orderId);
        List<TourOrderPassenage> tourOrderPassenageList = tourOrderPassenageService.getTourOrderPassenageListWithRelation(tourOrderPassenage, "");
        TourOrderVo tourOrderVo = MyModelUtil.copyTo(tourOder1, TourOrderVo.class);

        TourismProject tourismProject = new TourismProject();
        tourismProject.setId(tourOrderVo.getProjectId());
        TourismProject tourismProject1 = tourismProjectService.getOne(tourismProject);
        TourismProjectVo tourismProjectVo = MyModelUtil.copyTo(tourismProject1, TourismProjectVo.class);
        tourOrderVo.setTourismProjectVo(tourismProjectVo);
        List<TourOrderPassenageVo> tourOrderPassenageVoList = MyModelUtil.copyCollectionTo(tourOrderPassenageList, TourOrderPassenageVo.class);
        tourOrderVo.setDetailList(tourOrderPassenageVoList);
        return ResponseResult.success(tourOrderVo);
    }


}
