package com.tourism.webadmin.app.website.service.impl;


import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.page.PageMethod;
import com.tourism.common.additional.config.ApplicationConfig;
import com.tourism.common.additional.utils.MapConvertUtils;
import com.tourism.common.additional.utils.StringUtils;
import com.tourism.common.additional.utils.UrlConvertUtils;
import com.tourism.common.core.annotation.MyRequestBody;
import com.tourism.common.core.constant.ErrorCodeEnum;
import com.tourism.common.core.object.*;
import com.tourism.common.core.upload.BaseUpDownloader;
import com.tourism.common.core.upload.UpDownloaderFactory;
import com.tourism.common.core.upload.UploadResponseInfo;
import com.tourism.common.core.upload.UploadStoreTypeEnum;
import com.tourism.common.core.util.IpUtil;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.core.util.MyPageUtil;
import com.tourism.webadmin.app.website.dto.TourismBookProjectDto;
import com.tourism.webadmin.app.website.dto.TourismProjectToWebDto;
import com.tourism.webadmin.app.website.service.CleanDataService;
import com.tourism.webadmin.app.website.service.TourismProjectToWebService;
import com.tourism.webadmin.app.website.vo.DateRange;
import com.tourism.webadmin.app.website.vo.TourismProjectDatePriceVo;
import com.tourism.webadmin.app.website.vo.WebSiteProjectDatePriceVo;
import com.tourism.webadmin.back.dao.JobContentMapper;
import com.tourism.webadmin.back.dao.TourTourismTravelNotesContentMapper;
import com.tourism.webadmin.back.dao.TourismContentMapper;
import com.tourism.webadmin.back.dto.MultipartFileDto;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.vo.TourismProjectVo;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.redisson.api.RBucket;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;

/**
 * 门户网站首页Service
 *
 * @author tourism
 */
@Slf4j
@Service("TourismProjectToWebService")
public class TourismProjectToWebServiceImpl implements TourismProjectToWebService {

    @Autowired
    private TourismProjectService tourismProjectService;
    @Autowired
    private ApplicationConfig applicationConfig;
    @Autowired
    private TourismDatePriceService tourismDatePriceService;
    @Autowired
    private TourUserService tourUserService;
    @Autowired
    private TourBookInfoService tourBookInfoService;
    @Autowired
    private RedissonClient redissonClient;
    @Autowired
    private DirectoryInfoService directoryInfoService;


    @Override
    public MyPageData<TourismProjectVo> list(TourismProjectToWebDto tourismProjectDtoFilter) {
//        if(tourismProjectDtoFilter.getBelongTab() == null){
//            return ResponseResult.error(ErrorCodeEnum.ARGUMENT_NULL_EXIST,"所属分类(belongTab)不能为空!");
//        }
        //如果belongTab小于1000并且大于10.则表明查询的是一级菜单
//        if(tourismProjectDtoFilter.getBelongTab() !=null && tourismProjectDtoFilter.getBelongTab()>=10 && tourismProjectDtoFilter.getBelongTab()<1000){
//            DirectoryInfo filter = new DirectoryInfo();
//            filter.setParentId(tourismProjectDtoFilter.getBelongTab());
//            filter.setEnable(1);
//            List<DirectoryInfo> directoryInfoList = tourismProjectService.getDirectoryInfoList(filter, null);
//            tourismProjectDtoFilter.setBelongTab(null);
//            tourismProjectDtoFilter.setDirectoryInfoIds(directoryInfoList.stream().map(DirectoryInfo::getId).collect(toList()));
//        }

        if(StringUtils.isEmpty(tourismProjectDtoFilter.getAreaId()) && StringUtils.isEmpty(tourismProjectDtoFilter.getCountryId())){
            //判断是否为全部的地区
        }else if(StringUtils.isNotEmpty(tourismProjectDtoFilter.getAreaId()) && StringUtils.isEmpty(tourismProjectDtoFilter.getCountryId())){
            //判断是否为该地区的所有国家
            DirectoryInfo directoryInfo = new DirectoryInfo();
            directoryInfo.setParentId(Long.parseLong(tourismProjectDtoFilter.getAreaId()));
            directoryInfo.setEnable(1);
            List<DirectoryInfo> directoryInfoList =
                    directoryInfoService.getDirectoryInfoList(directoryInfo, "");
            List<Long> directoryIdList =
                    directoryInfoList.stream().map(DirectoryInfo::getId).collect(Collectors.toList());
            tourismProjectDtoFilter.setDirectoryInfoIds(directoryIdList);
        }else {
            //如果查询的是某个国家的游记的话
            if(StringUtils.isNotEmpty(tourismProjectDtoFilter.getCountryId())) {
                tourismProjectDtoFilter.setBelongTab(Long.valueOf(tourismProjectDtoFilter.getCountryId()));
            }
        }
        TourismProject tourismProjectFilter = MyModelUtil.copyTo(tourismProjectDtoFilter, TourismProject.class);
        //首页展示的为启用的内容
        tourismProjectFilter.setEnable(1);
//        String orderBy = MyOrderParam.buildOrderBy(tourismProjectDtoFilter.getOrderParamList(), TourismProject.class);
        MyOrderParam myOrderParam = new MyOrderParam();
        myOrderParam.add(new MyOrderParam.OrderInfo("isHotspot", true, null));
        myOrderParam.add(new MyOrderParam.OrderInfo("showOrder", true, null));
        myOrderParam.add(new MyOrderParam.OrderInfo("updateTime", false, null));
        String orderBy = MyOrderParam.buildOrderBy(myOrderParam, TourismProject.class);
        if (tourismProjectDtoFilter.getPageNum() != null && tourismProjectDtoFilter.getPageSize() != null ) {
            PageMethod.startPage(tourismProjectDtoFilter.getPageNum(), tourismProjectDtoFilter.getPageSize(), true);
        }
        List<TourismProject> tourismProjectList =
                tourismProjectService.getTourismProjectList(tourismProjectFilter, orderBy);
//        List<TourismProject> tourismProjectList =
//                tourismProjectService.getTourismProjectListWithRelation(tourismProjectFilter, orderBy);
        MyPageData<TourismProjectVo> tourismProjectVoMyPageData = MyPageUtil.makeResponseData(tourismProjectList, TourismProjectVo.class);

        List<TourismProjectVo> dataList = tourismProjectVoMyPageData.getDataList();
        //先把imgUrl由jaon转换为List<FileUrlObject>
        dataList.stream().forEach(item ->{
            List<String> urlList = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getHomeHotPicture());
            item.setHomeHotPicturesAfterConvert(urlList);
            List<String> arrayList = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getTourismUrl());
            item.setTourismUrlsAfterConvert(arrayList);
            List<String> arrayList1 = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getTravelNotesBanner());
            item.setTravelNotesBannerAfterConvert(arrayList1);
        });
        return tourismProjectVoMyPageData;
    }

    @Override
    public TourismProjectVo detail(TourismProject tourismProject) {
        TourismProjectVo tourismProjectVo = MyModelUtil.copyTo(tourismProject, TourismProjectVo.class);
        List<String> arrayList = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), tourismProjectVo.getTourismUrl());
        tourismProjectVo.setTourismUrlsAfterConvert(arrayList);
        List<String> arrayList1 = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), tourismProjectVo.getHomeHotPicture());
        tourismProjectVo.setHomeHotPicturesAfterConvert(arrayList1);
        List<String> arrayList2 = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), tourismProjectVo.getTravelNotesBanner());
        tourismProjectVo.setTravelNotesBannerAfterConvert(arrayList2);
        if(tourismProjectVo.getTourismFile() != null){
            // 使用 Jackson 的 ObjectMapper 进行转换
            ObjectMapper objectMapper = new ObjectMapper();
            TourismFile tourismFile = objectMapper.convertValue(tourismProjectVo.getTourismFile(), TourismFile.class);
            tourismFile.setFileUrlsAfterConvert(UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), tourismFile.getFileUrl()));
            tourismProjectVo.setTourismFile(MapConvertUtils.convertObjectToMap(tourismFile));
        }
        //二维码路径赋值
//        tourismProjectVo.setContactCode("[{\"name\":\"微信图片_20241021154757.png\",\"downloadUri\":\"/admin/app/tourTourismProjectTravelNotes/download\",\"filename\":\"6b92b75edcc04da1bd6e4af056911730.png\",\"uploadPath\":\"image/TourTourismProjectTravelNotes/contactCode\"}]");
//        List<String> urlConvertList = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), tourismProjectVo.getContactCode());
//        if(CollectionUtils.isNotEmpty(urlConvertList)) {
//            tourismProjectVo.setContactCodeConvert(urlConvertList.get(0));
//        }
        if(tourismProjectVo != null) {
            tourismProjectVo.setContactCodeConvert("https://v.xiaoyaotravel.com/image/ContactQRCode/tourism.png");
        }
        return tourismProjectVo;
    }

    @Override
    public WebSiteProjectDatePriceVo view(String projectId) {

        TourismDatePrice tourismDatePrice = new TourismDatePrice();
        tourismDatePrice.setProjectId(projectId);
        tourismDatePrice.setNowDate(new Date());

        //查询进行排序
        MyOrderParam myOrderParam = new MyOrderParam();
        MyOrderParam.OrderInfo orderInfo = new MyOrderParam.OrderInfo();
        orderInfo.setFieldName("departureDate");
        orderInfo.setAsc(true);
        myOrderParam.add(orderInfo);
        String orderBy = MyOrderParam.buildOrderBy(myOrderParam, TourismDatePrice.class);
        List<TourismDatePrice> tourismDatePriceList =
                tourismDatePriceService.getTourismDatePriceList(tourismDatePrice, orderBy);
//        List<TourismDatePrice> tourismDatePriceSortList = tourismDatePriceList.stream()
//                .sorted(Comparator.comparing(TourismDatePrice::getDepartureDate))
//                .collect(Collectors.toList());
        List<TourismProjectDatePriceVo> tourismProjectDatePriceVos = MyModelUtil.copyCollectionTo(tourismDatePriceList, TourismProjectDatePriceVo.class);

        DateRange dateRange = new DateRange();
        dateRange.setStartDate(new Date().toInstant().atZone(ZoneId.systemDefault()).toLocalDate());
        if(!CollectionUtils.isEmpty(tourismDatePriceList)) {
            Optional<Date> max = tourismDatePriceList.stream()
                    .map(TourismDatePrice::getDepartureDate)
                    .max(Comparator.naturalOrder());
            dateRange.setEndDate(max.get().toInstant().atZone(ZoneId.systemDefault()).toLocalDate());
        }else {
            dateRange.setEndDate(DateUtils.addDays(new Date(), 30).toInstant().atZone(ZoneId.systemDefault()).toLocalDate());
        }
//        //查询项目的默认价格
//        TourismProject tourismProject = tourismProjectService.getById(projectId);
//        if(tourismProject != null){
//            tourismProjectDatePriceVos.stream().forEach(item->{
//                item.setProjectPrice(tourismProject.getPrice().toString().concat(tourismProject.getPriceUnit()));
//            });
//        }
        WebSiteProjectDatePriceVo webSiteProjectDatePriceVo = new WebSiteProjectDatePriceVo();
        // 使用 Stream API 将列表转换为 Map
        Map<LocalDate, TourismProjectDatePriceVo> map = tourismProjectDatePriceVos.stream()
                .collect(Collectors.toMap(
                        TourismProjectDatePriceVo::getDepartureDate, // key
                        vo -> vo // value
                ));
        webSiteProjectDatePriceVo.setTourismProjectDatePriceVos(map);
        webSiteProjectDatePriceVo.setDateRange(dateRange);
        return webSiteProjectDatePriceVo;
    }

    @Override
    public Boolean bookProject(HttpServletRequest request, TourismBookProjectDto tourBookInfoDto) {

        //获取用户的手机号
        Long userId = TokenData.takeFromRequest().getUserId();
        TourUser tourUser = tourUserService.getById(userId);
        String mobile = tourUser.getMobile();

        RBucket<Object> bucket = redissonClient.getBucket(tourBookInfoDto.getString().concat(mobile));
        if(bucket.get() != null){
            return false;
        }

        if(tourBookInfoDto == null){
            throw new RuntimeException("预定参数为空！");
        }

        //根据当前日期，查询当天的项目日历价格
        TourismDatePrice tourismDatePrice = new TourismDatePrice();
        tourismDatePrice.setProjectId(tourBookInfoDto.getProjectId());
        tourismDatePrice.setDepartureDate(tourBookInfoDto.getStartDate());
        TourismDatePrice tourismDatePriceOne = tourismDatePriceService.getOne(tourismDatePrice);


        //构建预定的数据进行保存
        TourBookInfo tourBookInfo = new TourBookInfo();
        tourBookInfo.setType(tourBookInfoDto.getType());
        tourBookInfo.setProjectId(Long.parseLong(tourBookInfoDto.getProjectId()));
        tourBookInfo.setAdultNumber(tourBookInfoDto.getAdultNumber());
        tourBookInfo.setChildrenNumber(tourBookInfoDto.getChildrenNumber());
        tourBookInfo.setAdultPrice(tourismDatePriceOne.getAdultPrice());
        tourBookInfo.setChildrenPrice(tourismDatePriceOne.getChildrenPrice());
        tourBookInfo.setBookMobile(mobile);
        tourBookInfo.setBookName(mobile);
        //默认初始化状态为0未处理
        tourBookInfo.setIsHandle(0);
        tourBookInfo.setBookTime(tourBookInfoDto.getStartDate());
        tourBookInfo.setTotalPrice( tourismDatePriceOne.getAdultPrice().multiply(BigDecimal.valueOf(tourBookInfoDto.getAdultNumber()))
                .add(tourismDatePriceOne.getChildrenPrice().multiply(BigDecimal.valueOf(tourBookInfoDto.getChildrenNumber()))));
        tourBookInfo.setBookIp(IpUtil.getRemoteIpAddress(request));
        tourBookInfoService.saveNew(tourBookInfo);
        bucket.set("预约成功!");
        return true;
    }
}
